##-----Replication of Results in Table 3--------
rm(list = ls())
setwd("insert folder")
source("sur.bma.R")
XX=read.csv("Table3.csv", header=TRUE) 

########################################################################################################
##-----Variable selection for runs: uncomment desired specification (Default: specification 8)--------##
########################################################################################################
# Dependent variable
Y.lab <- "g"
# Instruments
Z.lab <- c("lland","landlocked","lpop","lagy0","laggpop","lagBLeduc","laginv","lagtotalgv","lagopres",
           "lagdp","easrel1900","hindu1900","jews1900","muslim1900","orth1900","othrel1900",
           "prot1900","lagexconst","lowland","lowlland","lowpop","lowmedland","lowmedlland","lowmedpop",
           "upmedland","upmedlland","upmedpop")
# Exogenous variables
W.lab <- c("low","lowmed","upmed","lifee1r", "fertl","easia","ssafr","lac","lcr100km", "kgatrstr",
           "language", "ethnic","exprsk","KKZ","intercept","legor_fr","legor_uk","dum7074","dum7579",
           "dum8084","dum8589","dum9094","dum9599","dum0004","dum0509")

# Endogenous variables
# Choice of diversity measures:
diversity <- "EM_cluster" # Alternative measures: "Gini","HHI","between_theil","within_theil","total_theil"
diversity.low <- "lowEM_cluster"  # Alternative measures: "lowGini","lowHHI","lowbetween_theil","lowwithin_theil","lowtotal_theil"
diversity.lowmed <- "lowmedEM_cluster" # Alternative measures: "lowEM_10","lowmedGini","lowmedHHI","lowmedbetween_theil","lowmedwithin_theil","lowmedtotal_theil"
diversity.upmed <- "upmedEM_cluster" # Alternative measures: "upmedGini","upmedHHI","upmedbetween_theil","upmedwithin_theil","upmedtotal_theil"

X.lab <- c(diversity,diversity.low,diversity.lowmed,diversity.upmed,"y0","gpop","BLeduc","inv","totalgv",
           "opres","dp","easrel","hindu","jews","muslim","orth","prot","othrel","exconst") 
########################################################################################################
########################################################################################################

##-----Preparations for IVBMA run--------
## Delete obs with missing data (use NA) and string variables
keep.var <- c(Y.lab,X.lab,Z.lab,W.lab)
keep.noNA <- rowSums(is.na(XX[,keep.var])) == 0
XX["countrycode"] <- NULL
XX["countryname"] <- NULL

## Standardize variables
XX_stand <- data.frame(scale(XX))
## Creating intercept
XX_stand[,ncol(XX_stand)] <- 1

## Collection of each variable's standard deviation
varscount <- nrow(data.frame(c(Y.lab,X.lab,W.lab)))
sdev <- data.frame(matrix(0, ncol = varscount, nrow = 1))
colnames(sdev) <- c(Y.lab,X.lab,W.lab)
rownames(sdev) <- c("SD")

for(i in names(sdev)){
  sdev[1,i] <- sd(XX[,i], na.rm = FALSE)
}
sdev[1,"intercept"] <- 1

Y <- XX_stand[keep.noNA, Y.lab]
X <- XX_stand[keep.noNA, X.lab]
W <- XX_stand[keep.noNA, W.lab]
Z <- XX_stand[keep.noNA, Z.lab]

n.obs <- length(Y)
print(paste("There were",n.obs,"observations"))

Y.all <- cbind(Y,X)
U <- list()
U[[1]] <- cbind(X,W) 

for(i in 2:dim(Y.all)[2])
{
  U[[i]] <- cbind(Z,W)
}

## Specify MCMC draws
s <- 3e4
##b <- round(s/10) ## the burn in will default to this, otherwise specify here

## Prior variable inclusion probability matrix
pi.M <- list() ## This is going to be a list with the same length as the Y.all matrix
for(i in 1:length(U)) pi.M[[i]] <- rep(.5, dim(U[[i]])[2]) ##Prior probability is set to .5 for all variables

##-----Now we run IVBMA-------- 
a <- sur.bma(Y.all,U, s = s,pi.M = pi.M)

##-----Generating result table-------- 
U.lab <- list() ## Just bookeeping
U.lab[[1]] <- c(names(X), names(W))
for(i in 2:dim(Y.all)[2])U.lab[[i]] <- c(names(Z), names(W))
helper <- sur.bma.summary(a, U.lab)
tbl=(helper[[1]])
sd_g=sdev[1,1]
sd_vars=sdev[1,2:varscount]
for(i in names(sd_vars)){
  tbl[i,5] <- (sd_g/sd_vars[1,i])*tbl[i,5]
  tbl[i,6] <- (sd_g/sd_vars[1,i])*tbl[i,6]
  tbl[i,7] <- (sd_g/sd_vars[1,i])*tbl[i,7]
  tbl[i,8] <- (sd_g/sd_vars[1,i])*tbl[i,8]
}
print(tbl)

##-----Bayesian Sargan instrument validity test-------- 
b <- sur.bma.iv.diagnostics(a, as.matrix(Z))
mean(b$BTIV.collapsed.cond)  ## 1 indicates problem, 0 indicates not.


##-----Reporting diversity interaction effects-------- 
## Diversity: low income countries
b.diversitylow <- (sd_g/sd_vars[1,diversity])*a$beta[[1]][,1] + (sd_g/sd_vars[1,diversity.low])*a$beta[[1]][,2]
## PIP
print(mean(abs(a$beta[[1]][,1]+a$beta[[1]][,2]) > 0))
## Conditional Mean
print(mean(b.diversitylow[abs(b.diversitylow)>0]))
## Conditional SD
print(sd(b.diversitylow[abs(b.diversitylow)>0]))

## Diversity: lower medium income countries
b.diversitylowmed <- (sd_g/sd_vars[1,diversity])*a$beta[[1]][,1] + (sd_g/sd_vars[1,diversity.lowmed])*a$beta[[1]][,3]
## PIP
print(mean(abs(a$beta[[1]][,1]+a$beta[[1]][,3]) > 0))
## Conditional Mean
print(mean(b.diversitylowmed[abs(b.diversitylowmed)>0]))
## Conditional SD
print(sd(b.diversitylowmed[abs(b.diversitylowmed)>0]))

## Diversity: upper medium income countries
b.diversityupmed <- (sd_g/sd_vars[1,diversity])*a$beta[[1]][,1] + (sd_g/sd_vars[1,diversity.upmed])*a$beta[[1]][,4]
## PIP
print(mean(abs(a$beta[[1]][,1]+a$beta[[1]][,4]) > 0))
## Conditional Mean
print(mean(b.diversityupmed[abs(b.diversityupmed)>0]))
## Conditional SD
print(sd(b.diversityupmed[abs(b.diversityupmed)>0]))
