##-----Replication of Results in Table 6--------
rm(list = ls())
setwd("insert folder")
source("sur.bma.R")
XX=read.csv("Table6.csv", header=TRUE) 

########################################################################################################
##-----Variable selection for runs: uncomment desired specification (Default: specification 20)-------##
########################################################################################################
# Dependent variable
Y.lab <- "g"

# Diversity-income interactions with openness (or primary export shares)
diversity.inter <- "EM_primaryexp" # Alternative measure: "EM_opres"
diversity.inter.low <- "lowEM_primaryexp"  # Alternative measure: "lowEM_opres"
diversity.inter.lowmed <- "lowmedEM_primaryexp" # Alternative measure: "lowmedEM_opres"
diversity.inter.upmed <- "upmedEM_primaryexp" # Alternative measure: "upmedEM_opres"

# Income interaction with openness (or primary export shares)
inter <- "primaryexp" # Alternative measure: "opres"
inter.low <- "low_primaryexp"  # Alternative measure: "low_opres"
inter.lowmed <- "lowmed_primaryexp"  # Alternative measure: "lowmed_opres"
inter.upmed <- "upmed_primaryexp"  # Alternative measure: "upmed_opres"

##-----Specification 20--------
# Endogenous variables
X.lab <- c(diversity.inter,diversity.inter.low,diversity.inter.lowmed,diversity.inter.upmed,"EM_j","lowEM",
           "lowmedEM","upmedEM","y0","gpop","BLeduc","inv","totalgv",inter,inter.low,inter.lowmed,
           inter.upmed,"dp","easrel","hindu","jews","muslim","orth","prot","othrel","exconst") 
# Instruments
Z.lab <- c("lland","landlocked","lpop","lagy0","laggpop","lagBLeduc","laginv","lagtotalgv","lagopres",
           "lagdp","easrel1900","hindu1900","jews1900","muslim1900","orth1900","othrel1900",
           "prot1900","lagexconst","lowland","lowlland","lowpop","lowmedland","lowmedlland","lowmedpop",
           "upmedland","upmedlland","upmedpop","lland_lagopres","lpop_lagopres","landlocked_lagopres",
           "lowlland_lagopres","lowlpop_lagopres","lowlandlocked_lagopres","lowmedlland_lagopres",
           "lowmedlpop_lagopres","lowmedlandlocked_lagopres","upmedlland_lagopres","upmedlpop_lagopres",
           "upmedlandlocked_lagopres","low_lagopres","lowmed_lagopres","upmed_lagopres")

# Exogenous variables
W.lab <- c("low","lowmed","upmed","lifee1r", "fertl", "easia","ssafr","lac","lcr100km", "kgatrstr",
           "language", "ethnic","exprsk","KKZ","intercept","legor_fr","legor_uk","dum7074","dum7579",
           "dum8084","dum8589","dum9094","dum9599","dum0004","dum0509")

##-----Specification 21--------
# Endogenous variables
# X.lab <- c(diversity.inter,diversity.inter.low,diversity.inter.lowmed,diversity.inter.upmed,"EM_j","lowEM",
#            "lowmedEM","upmedEM","y0","gpop","BLeduc","inv","totalgv","opres","dp","easrel","hindu","jews",
#            "muslim","orth","prot","othrel","exconst") 
# 
# # Instruments
# Z.lab <- c("lland","landlocked","lpop","lagy0","laggpop","lagBLeduc","laginv","lagtotalgv","lagopres",
#            "lagdp","easrel1900","hindu1900","jews1900","muslim1900","orth1900","othrel1900",
#            "prot1900","lagexconst","lowland","lowlland","lowpop","lowmedland","lowmedlland","lowmedpop",
#            "upmedland","upmedlland","upmedpop")
# 
# # Exogenous variables
# W.lab <- c(inter,inter.low,inter.lowmed,inter.upmed,"low","lowmed","upmed",
#            "lifee1r", "fertl", "easia","ssafr","lac","lcr100km", "kgatrstr","language", "ethnic",
#            "exprsk","KKZ","intercept","legor_fr","legor_uk","dum7074","dum7579","dum8084","dum8589",
#            "dum9094","dum9599","dum0004","dum0509")
########################################################################################################
########################################################################################################

##-----Preparations for IVBMA run--------
## delete obs with missing data (use NA)
keep.var <- c(Y.lab,X.lab,Z.lab,W.lab)
keep.noNA <- rowSums(is.na(XX[,keep.var])) == 0
XX["countrycode"] <- NULL
XX["countryname"] <- NULL

## Standardize variables:
XX_stand <- data.frame(scale(XX))
## Creating intercept
XX_stand[,ncol(XX_stand)] <- 1

## Collection of each variable's standard deviation
varscount <- nrow(data.frame(c(Y.lab,X.lab,W.lab)))
sdev <- data.frame(matrix(0, ncol = varscount, nrow = 1))
mean <- data.frame(matrix(0, ncol = varscount, nrow = 1))
colnames(sdev) <- c(Y.lab,X.lab,W.lab)
rownames(sdev) <- c("SD")
colnames(mean) <- c(Y.lab,X.lab,W.lab)
rownames(mean) <- c("Mean")

for(i in names(sdev)){
  sdev[1,i] <- sd(XX[,i], na.rm = FALSE)
}
sdev[1,"intercept"] <- 1

for(i in names(mean)){
  mean[1,i] <- mean(XX[,i], na.rm = FALSE)
}
mean[1,"intercept"] <- 1

Y <- XX_stand[keep.noNA, Y.lab]
X <- XX_stand[keep.noNA, X.lab]
W <- XX_stand[keep.noNA, W.lab]
Z <- XX_stand[keep.noNA, Z.lab]

n.obs <- length(Y)
print(paste("There were",n.obs,"observations"))

Y.all <- cbind(Y,X)
U <- list()
U[[1]] <- cbind(X,W) 

for(i in 2:dim(Y.all)[2])
{
  U[[i]] <- cbind(Z,W)
}

## Specify MCMC draws
s <- 3e4
##b <- round(s/10) ## the burn in will default to this, otherwise specify here

## Prior variable inclusion probability matrix
pi.M <- list() ## This is going to be a list with the same length as the Y.all matrix
for(i in 1:length(U)) pi.M[[i]] <- rep(.5, dim(U[[i]])[2]) ##Prior probability is set to .5 for all variables

##-----Now we run IVBMA-------- 
a <- sur.bma(Y.all,U, s = s,pi.M = pi.M)


##-----Generating result table-------- 
U.lab <- list() ## Just bookeeping
U.lab[[1]] <- c(names(X), names(W))
for(i in 2:dim(Y.all)[2])U.lab[[i]] <- c(names(Z), names(W))
helper <- sur.bma.summary(a, U.lab)
tbl=(helper[[1]])
sd_g=sdev[1,1]
sd_vars=sdev[1,2:varscount]
mean_vars=mean[1,2:varscount]
for(i in names(sd_vars)){
  tbl[i,5] <- (sd_g/sd_vars[1,i])*tbl[i,5]
  tbl[i,6] <- (sd_g/sd_vars[1,i])*tbl[i,6]
  tbl[i,7] <- (sd_g/sd_vars[1,i])*tbl[i,7]
  tbl[i,8] <- (sd_g/sd_vars[1,i])*tbl[i,8]
}
print(tbl)


##-----Bayesian Sargan instrument validity test-------- 
b <- sur.bma.iv.diagnostics(a, as.matrix(Z))
mean(b$BTIV.collapsed.cond)  ## 1 indicates problem, 0 indicates not.


##-----Reporting diversity interaction effects--------
## Diversity : low income countries
b.EMlow <- (sd_g/sd_vars[1,"EM_j"])*a$beta[[1]][,5] + (sd_g/sd_vars[1,"lowEM"])*a$beta[[1]][,6] +
           (sd_g/sd_vars[1,diversity.inter])*a$beta[[1]][,1]*mean_vars[1,inter] + (sd_g/sd_vars[1,diversity.inter.low])*a$beta[[1]][,2]*mean_vars[1,inter]
## PIP
print(mean(abs(a$beta[[1]][,5]+a$beta[[1]][,6]+a$beta[[1]][,1]+a$beta[[1]][,2]) > 0))
## Conditional Mean
print(mean(b.EMlow[b.EMlow>0]))
## Conditional SD
print(sd(b.EMlow[b.EMlow>0]))

## Diversity (extensive margin): lower medium income countries
b.EMlowmed <- (sd_g/sd_vars[1,"EM_j"])*a$beta[[1]][,5] + (sd_g/sd_vars[1,"lowmedEM"])*a$beta[[1]][,7] +
  (sd_g/sd_vars[1,diversity.inter])*a$beta[[1]][,1]*mean_vars[1,inter] + (sd_g/sd_vars[1,diversity.inter.lowmed])*a$beta[[1]][,3]*mean_vars[1,inter]
## PIP
print(mean(abs(a$beta[[1]][,5]+a$beta[[1]][,7]+a$beta[[1]][,1]+a$beta[[1]][,3]) > 0))
## Conditional Mean
print(mean(b.EMlowmed[b.EMlowmed>0]))
## Conditional SD
print(sd(b.EMlowmed[b.EMlowmed>0]))

## Diversity (extensive margin): upper medium income countries
b.EMupmed <- (sd_g/sd_vars[1,"EM_j"])*a$beta[[1]][,5] + (sd_g/sd_vars[1,"upmedEM"])*a$beta[[1]][,8] +
  (sd_g/sd_vars[1,diversity.inter])*a$beta[[1]][,1]*mean_vars[1,inter] + (sd_g/sd_vars[1,diversity.inter.upmed])*a$beta[[1]][,4]*mean_vars[1,inter]
## PIP
print(mean(abs(a$beta[[1]][,5]+a$beta[[1]][,8]+a$beta[[1]][,1]+a$beta[[1]][,4]) > 0))
## Conditional Mean
print(mean(b.EMupmed[b.EMupmed>0]))
## Conditional SD
print(sd(b.EMupmed[b.EMupmed>0]))




