rwish <- function (delta, D) 
{
  ##Bartlet Decomposition
  T <- chol(solve(D))
  p <- dim(D)[1]
  Psi <- matrix(0, p, p)
  for (i in 1:p) Psi[i, i] <- sqrt(rchisq(1, delta + p - i))
  if (p > 1)
    {
      for (i in 1:(p - 1))
        {
          for (j in (i + 1):p)
            {
              Psi[i, j] <- rnorm(1)
            }
        }
    }
  Psi <- Psi %*% T
  return(t(Psi) %*% Psi)
}

rmvnorm.precision <- function(mu,K)
  {
    p <- dim(K)[2]
    Q <- chol(K)
    z <- rnorm(p)
    x <- backsolve(Q,z) + mu
    return(x)
  }

mydet <- function(A)
  {
    return(2 * sum(log(diag(chol(A)))))
  }



sur.bma.update.r <- function(theta,D,r)
  {

    R <- D$R
    ##------ Stoopid ----------------
    l <- NULL
    l$M <- theta$M[[r]]
    l$beta <- theta$beta[[r]]
    Y <- D$Y[,r]
    U <- D$U[[r]]
    K <- theta$K
    for(i in 1:R)
      {
        if(i != r)
          {
            Y <- Y + K[i,r]/K[r,r] * (D$Y[,i] - D$U[[i]] %*% theta$beta[[i]])
          }
      }

    Y <- sqrt(theta$alpha) * Y
    
    ##------ Keep book --------
    n <- length(Y)
    p.U <- dim(U)[2]
    M <- l$M
    ##------------------------

    ##----- Flip a variable -----
    pp <- (1 - M) * D$pi.M[[r]] + M * (1 - D$pi.M[[r]])
    w <- sample(1:p.U,1,prob= pp)
    M.new <- M
    M.new[w] <- 1 - M.new[w]
    if(sum(M.new) == 0) return(l)
    ##-----------------------------
    
    ##---- New Score ------------
    p.2 <- sum(M.new)
    U.2 <- U[,(1:p.U)[M.new == 1],drop=FALSE]
    for(i in 1:n)
      {
        U.2[i,] <-sqrt(theta$alpha[i]) * U.2[i,]
      }
    Omega.2 <- diag(p.2) + K[r,r] * t(U.2) %*% U.2
    lambda.2 <- K[r,r] * t(Y) %*% as.matrix(U.2) %*% solve(Omega.2)
    score.2 <- 0.5 * lambda.2 %*% Omega.2 %*% t(lambda.2) - 0.5 * mydet(Omega.2)
    ##---------------------------

    ##---- Old Score ------------
    p.1 <- sum(M)
    U.1 <- U[,(1:p.U)[M == 1],drop=FALSE]
    for(i in 1:n)
      {
        U.1[i,] <- sqrt(theta$alpha[i]) * U.1[i,]
      }
    Omega.1 <- diag(p.1) + K[r,r] * t(U.1) %*% U.1
    lambda.1 <- K[r,r] * t(Y) %*% U.1 %*% solve(Omega.1)
    score.1 <- 0.5 * lambda.1 %*% Omega.1 %*% t(lambda.1) - 0.5 * mydet(Omega.1)
    ##---------------------------

    ##---- Decide ---------------
    alpha <- score.2 - score.1
    if(log(runif(1)) < alpha)
      {
        M <- M.new
        Omega.1 <- Omega.2
        lambda.1 <- lambda.2
      }
    ##---------------------------

    beta.1 <- rep(0,p.U)
    beta.1[ (1:p.U)[M==1]] <- rmvnorm.precision(lambda.1,Omega.1)
    l$M <- M
    l$beta <- beta.1

    return(l)
  }

sur.bma.update.alpha <- function(theta,D)
  {
    n <- D$n
    alpha <- rep(0,n)
    K <- theta$K
    eps <- matrix(0, D$n, D$R)
    R <- D$R
    for(r in 1:R)
      {
        eps[,r] <- D$Y[,r] - D$U[[r]] %*% theta$beta[[r]]
      }

    alpha[1] <- 1
    for(i in 2:n)
      {
        s <- eps[i,] %*% K %*% eps[i,]
        alpha[i] <- rgamma(1,(theta$nu + R)/2, (theta$nu + s)/2)
      }
    return(alpha)
  }

sur.bma.update.nu <- function(theta)
  {
    n <- length(theta$alpha)
    nu <- theta$nu
    alpha <- theta$alpha
    nu.prime <- rtnorm(1,nu,1,lower = 0)

    score.lik.top <- sum(dgamma(alpha,nu.prime/2,nu.prime/2,log=TRUE))
    score.prior.top <- dtnorm(nu.prime, 5,sqrt(100),lower = 0, log = TRUE)
    score.prop.top <- dtnorm(nu, nu.prime,1,lower = 0, log = TRUE)

    score.lik.bot <- sum(dgamma(alpha,nu/2,nu/2,log=TRUE))
    score.prior.bot <- dtnorm(nu, 5, sqrt(100), lower = 0, log = TRUE)
    score.prop.bot <- dtnorm(nu.prime, nu,1,lower = 0, log = TRUE)

    mh <- score.lik.top + score.prior.top + score.prop.top - score.lik.bot - score.prior.bot - score.prop.bot
    
    if(log(runif(1)) < mh)
      {
        return(nu.prime)
      }else{
        return(nu)
      }
  }

sur.bma.update <- function(theta, D,full)
  {
    n <- D$n
    eps <- matrix(0, D$n, D$R)
    R <- D$R
    for(r in 1:R)
      {
        l <- sur.bma.update.r(theta,D,r)
        theta$M[[r]] <- l$M
        theta$beta[[r]] <- l$beta
        eps[,r] <- D$Y[,r] - D$U[[r]] %*% theta$beta[[r]]
      }

    if(theta$dispersion)
      {
        for(i in 1:n)
          {
            eps[i,] <- sqrt(theta$alpha[i]) * eps[i,]
          }
      }

    U <- (D$n - 1) * cov(eps)
    theta$K <- rwish(D$n + 3, diag(R) + U)
    
    if(theta$dispersion)
      {
        theta$alpha <- sur.bma.update.alpha(theta,D)
        theta$nu <- sur.bma.update.nu(theta)
      }


    return(theta)
  }

sur.bma.init <- function(D,full,dispersion)
  {

    theta <- NULL
    theta$beta <- list()
    theta$M <- list()
    for(r in 1:D$R)
      {
        p.U.r <- dim(D$U[[r]])[2]
        theta$beta[[r]] <- solve(t(D$U[[r]]) %*% D$U[[r]]) %*% t(D$U[[r]]) %*% D$Y[,r]
        
        if(full)
          {
            theta$M[[r]] <- 1
          }else{
            theta$M[[r]] <- rbinom(p.U.r,1,D$pi.M[[r]])
          }
        theta$beta[[r]][theta$M[[r]] == 0] <- 0
      }
    theta$K <- diag(D$R)
    theta$G <- matrix(1,D$R,D$R)
    theta$alpha <- rep(1,D$n)
    theta$nu <- rnorm(1,20,.1)
    theta$dispersion <- dispersion
    return(theta)
  }

sur.bma.results.init <- function(D,odens)
  {

    R <- D$R
    n <- D$n
    ##-------- Information to be returned ----------
    results <- NULL
    results$beta <- list()
    results$beta.bar <- list()
    results$M <- list()
    results$M.bar <- list()
    for(r in 1:R)
      {
        p.U.r <- dim(D$U[[r]])[2]
        results$beta[[r]] <- matrix(0, odens, p.U.r)
        results$beta.bar[[r]]  <- rep(0,p.U.r)
        results$M[[r]] <- matrix(0, odens, p.U.r)
        results$M.bar[[r]]  <- rep(0,p.U.r)
      }
    results$K <- array(dim = c(R,R,odens))
    results$G <- array(dim = c(R,R,odens))
    results$K.bar <- matrix(0,R,R)
    results$G.bar <- matrix(0,R,R)
    results$nu <- rep(0, odens)
    results$nu.bar <- 0
    results$y.hat <- array(dim = c(n,odens,R))
    results$y.hat.bar <- matrix(0,n,R)
    ##--------------------------------------------
    
    return(results)
  }

sur.bma <- function(Y, U, s, b = round(s/10),
                    full = FALSE,odens = max(c(5e3,s-b)),
                    pi.M = NULL,
                    print.every = round(s/10),
                    dispersion = FALSE)
  {
    D <- NULL
    D$Y <- as.matrix(Y)
    D$n <- dim(D$Y)[1]
    D$R <- length(U)
    D$U <- list()
    R <- D$R
    for(r in 1:R) D$U[[r]] <- as.matrix(U[[r]])
    if(is.null(pi.M))
      {
        D$pi.M <- list()
        for(r in 1:R) D$pi.M[[r]] <- rep(.5,dim(U[[r]])[2])
      }else{
        D$pi.M <- pi.M
      }
    
    theta <- sur.bma.init(D,full,dispersion)
    results <- sur.bma.results.init(D,odens)
    which.save <- round(seq(b + 1, s, length = odens))
    save.loc <- 1
    next.save <- which.save[save.loc]

    for(i in 1:s)
      {

        if(i %% print.every == 0)print(i)
        theta <- sur.bma.update(theta,D)
##        print(c(round(theta$nu, 3), round(var(theta$alpha),3), round(mean(theta$alpha),3)))
        ##---------- Record ----------------------------
        if(i == next.save)
          {
            for(r in 1:R)
              {
                results$M[[r]][save.loc,] <- theta$M[[r]]
                results$beta[[r]][save.loc,] <- theta$beta[[r]]
                results$K[,,save.loc] <- theta$K
                results$G[,,save.loc] <- theta$G
                results$nu[save.loc] <- theta$nu
                results$y.hat[,save.loc,r] <- D$U[[r]] %*% theta$beta[[r]]
              }
            save.loc <- save.loc + 1
            next.save <- which.save[save.loc]
          }
        if(i > b)
          {
            for(r in 1:R)
              {
                results$M.bar[[r]] <- results$M.bar[[r]] + theta$M[[r]]/ (s - b)
                results$beta.bar[[r]] <- results$beta.bar[[r]] + theta$beta[[r]]/ (s - b)
                results$y.hat.bar[,r] <- results$y.hat.bar[,r] + (D$U[[r]] %*% theta$beta[[r]])/(s-b)
              }
            results$K.bar <- results$K.bar + theta$K / (s - b)
            results$G.bar <- results$G.bar + theta$G / (s - b)
            results$nu.bar <- results$nu.bar + theta$nu / (s- b)
          }
        ##------------------------------------------------------
      }
    results$D <- D
    return(results)
  }


sur.bma.summary <- function(a,U.lab)
  {

    tbl <- list()
    for(r in 1:length(a$beta))
      {
        tbl[[r]] <- cbind(a$M.bar[[r]], t(apply(a$beta[[r]],2,"quantile",c(.025,.5, .975))),a$beta.bar[[r]], apply(a$beta[[r]],2,sd))
        A <- NULL
        B <- NULL
        for(k in 1:dim(tbl[[r]])[1])
          {
            A[k] <- sd(a$beta[[r]][a$beta[[r]][,k] != 0,k])
            B[k] <- mean(a$beta[[r]][a$beta[[r]][,k] != 0,k])
          }
        tbl[[r]] <- cbind(tbl[[r]], cbind(B,A))
      }
    

                                        #changed rownames(tbl[[1]]) <- c(X.lab,"X1.sq","Intercept",W.lab)
    for(r in 1:length(U.lab))
      {
        rownames(tbl[[r]]) <- U.lab[[r]]
        colnames(tbl[[r]]) <- c("Prob","Low","Med","Up","Mean","SD","CondMean","CondSD")
      }

    return(tbl)
  }

## a is an object returned from sur.bma
## I assume the first element in Y and U is the dependent equation
## which.z is a list of which objects in the remaining covariates are instruments
sur.bma.iv.diagnostics <- function(a, Z)
  {
    results <- NULL
    results$BTIV.collapsed <- NULL
    results$BTIV.collapsed.cond <- NULL
    q <- dim(Z)[2]
    n.0 <- 1
    n <- dim(a$D$Y)[1]
    R <- dim(a$D$Y)[2]
    eps <- matrix(0,n,R)
    
    for(i in 1:dim(a$beta[[1]])[1])
        {
          K <- a$K[,,i]
          for(r in 1:dim(a$D$Y)[2])eps[,r] <- (a$D$Y[,r] - a$D$U[[r]] %*% a$beta[[r]][i,])
          eps.hat <- eps[,1]
          eps.cond <- eps[,1] + eps[,2:R] %*% K[1,2:R]/K[1,1]
          K.Z <- (n.0 * diag(q) + t(Z) %*% Z)
          beta.Z <- solve(K.Z) %*% t(Z) %*% eps.hat
          S.Z <- t(eps.hat) %*% eps.hat - t(beta.Z) %*% K.Z %*% beta.Z
          score.top <- (q/2) * log(n.0) - 1/2 * mydet(K.Z) - (1 + n)/2 * log((1/2 + S.Z/2))
          score.bottom <- -(1 + n)/2 * log((1/2 + t(eps.hat) %*% eps.hat/2))
          results$BTIV.collapsed[i] <- 1/(1 + exp(score.bottom - score.top))

          beta.Z <- solve(K.Z) %*% t(Z) %*% eps.cond
          S.Z <- t(eps.cond) %*% eps.cond - t(beta.Z) %*% K.Z %*% beta.Z
          score.top <- (q/2) * log(n.0) - 1/2 * mydet(K.Z) - (1 + n)/2 * log((1/2 + S.Z/2))
          score.bottom <- -(1 + n)/2 * log((1/2 + t(eps.cond) %*% eps.cond/2))
          results$BTIV.collapsed.cond[i] <- 1/(1 + exp(score.bottom - score.top))
        }
        return(results)

  }
